@@ -46,6 +46,7 @@
 #include <ofono/call-settings.h>
 #include <ofono/call-volume.h>
 #include <ofono/cbs.h>
+#include <ofono/ctm.h>
 #include <ofono/devinfo.h>
 #include <ofono/message-waiting.h>
 #include <ofono/netreg.h>
@@ -64,6 +65,7 @@
 #include <drivers/atmodem/atutil.h>
 
 static const char *none_prefix[] = { NULL };
+static const char *ptty_prefix[] = { "+PTTY:", NULL };
 static int next_iface = 0;
 
 struct phonesim_data {
@@ -141,9 +143,6 @@ static void phonesim_deactivate_primary(struct ofono_gprs_context *gc,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[128];
 
-	if (!cbd)
-		goto error;
-
 	cbd->user = gc;
 
 	snprintf(buf, sizeof(buf), "AT+CGACT=0,%u", id);
@@ -152,7 +151,6 @@ static void phonesim_deactivate_primary(struct ofono_gprs_context *gc,
 				at_cgact_down_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
@@ -165,7 +163,7 @@ static int phonesim_context_probe(struct ofono_gprs_context *gc,
 	struct gprs_context_data *gcd;
 
 	gcd = g_try_new0(struct gprs_context_data, 1);
-	if (!gcd)
+	if (gcd == NULL)
 		return -ENOMEM;
 
 	gcd->chat = g_at_chat_clone(chat);
@@ -190,12 +188,139 @@ static void phonesim_context_remove(struct ofono_gprs_context *gc)
 	g_free(gcd);
 }
 
+static void phonesim_ctm_support_cb(gboolean ok, GAtResult *result,
+					gpointer user_data)
+{
+	struct ofono_ctm *ctm = user_data;
+
+	if (!ok) {
+		ofono_ctm_remove(ctm);
+		return;
+	}
+
+	ofono_ctm_register(ctm);
+}
+
+static int phonesim_ctm_probe(struct ofono_ctm *ctm,
+				unsigned int vendor, void *data)
+{
+	GAtChat *chat;
+
+	DBG("");
+
+	chat = g_at_chat_clone(data);
+
+	ofono_ctm_set_data(ctm, chat);
+
+	g_at_chat_send(chat, "AT+PTTY=?", ptty_prefix, phonesim_ctm_support_cb,
+			ctm, NULL);
+
+	return 0;
+}
+
+static void phonesim_ctm_remove(struct ofono_ctm *ctm)
+{
+	GAtChat *chat = ofono_ctm_get_data(ctm);
+
+	DBG("");
+
+	ofono_ctm_set_data(ctm, NULL);
+
+	g_at_chat_unref(chat);
+}
+
+static void ctm_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	struct ofono_error error;
+	GAtResultIter iter;
+	ofono_ctm_query_cb_t cb = cbd->cb;
+	int value;
+
+	decode_at_error(&error, g_at_result_final_response(result));
+
+	if (!ok) {
+		cb(&error, -1, cbd->data);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	if (g_at_result_iter_next(&iter, "+PTTY:") == FALSE)
+		goto error;
+
+	if (g_at_result_iter_next_number(&iter, &value) == FALSE)
+		goto error;
+
+	cb(&error, value, cbd->data);
+
+	return;
+
+error:
+
+	CALLBACK_WITH_FAILURE(cb, -1, cbd->data);
+}
+
+static void phonesim_ctm_query(struct ofono_ctm *ctm,
+				ofono_ctm_query_cb_t cb, void *data)
+{
+	GAtChat *chat = ofono_ctm_get_data(ctm);
+	struct cb_data *cbd = cb_data_new(cb, data);
+
+	DBG("");
+
+	if (g_at_chat_send(chat, "AT+PTTY?", ptty_prefix,
+				ctm_query_cb, cbd, g_free) > 0)
+		return;
+
+	g_free(cbd);
+
+	CALLBACK_WITH_FAILURE(cb, 0, data);
+}
+
+static void ctm_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_ctm_set_cb_t cb = cbd->cb;
+	struct ofono_error error;
+
+	decode_at_error(&error, g_at_result_final_response(result));
+	cb(&error, cbd->data);
+}
+
+static void phonesim_ctm_set(struct ofono_ctm *ctm, ofono_bool_t enable,
+				ofono_ctm_set_cb_t cb, void *data)
+{
+	GAtChat *chat = ofono_ctm_get_data(ctm);
+	struct cb_data *cbd = cb_data_new(cb, data);
+	char buf[20];
+
+	DBG("");
+
+	snprintf(buf, sizeof(buf), "AT+PTTY=%d", enable);
+
+	if (g_at_chat_send(chat, buf, none_prefix,
+				ctm_set_cb, cbd, g_free) > 0)
+		return;
+
+	CALLBACK_WITH_FAILURE(cb, data);
+	g_free(cbd);
+}
+
 static struct ofono_gprs_context_driver context_driver = {
+	.name                   = "phonesim",
+	.probe                  = phonesim_context_probe,
+	.remove                 = phonesim_context_remove,
+	.activate_primary       = phonesim_activate_primary,
+	.deactivate_primary     = phonesim_deactivate_primary,
+};
+
+static struct ofono_ctm_driver ctm_driver = {
 	.name			= "phonesim",
-	.probe			= phonesim_context_probe,
-	.remove			= phonesim_context_remove,
-	.activate_primary	= phonesim_activate_primary,
-	.deactivate_primary	= phonesim_deactivate_primary,
+	.probe			= phonesim_ctm_probe,
+	.remove			= phonesim_ctm_remove,
+	.query_tty		= phonesim_ctm_query,
+	.set_tty		= phonesim_ctm_set,
 };
 
 static int phonesim_probe(struct ofono_modem *modem)
@@ -205,7 +330,7 @@ static int phonesim_probe(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	data = g_try_new0(struct phonesim_data, 1);
-	if (!data)
+	if (data == NULL)
 		return -ENOMEM;
 
 	ofono_modem_set_data(modem, data);
@@ -237,6 +362,30 @@ static void cfun_set_on_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_modem_set_powered(modem, ok);
 }
 
+static gboolean phonesim_reset(void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct phonesim_data *data = ofono_modem_get_data(modem);
+
+	g_at_chat_unref(data->chat);
+	data->chat = NULL;
+
+	if (data->mux) {
+		g_at_mux_shutdown(data->mux);
+		g_at_mux_unref(data->mux);
+		data->mux = NULL;
+	}
+
+	ofono_modem_reset(modem);
+
+	return FALSE;
+}
+
+static void crst_notify(GAtResult *result, gpointer user_data)
+{
+	g_idle_add(phonesim_reset, user_data);
+}
+
 static void phonesim_disconnected(gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
@@ -265,7 +414,7 @@ static void mux_setup(GAtMux *mux, gpointer user_data)
 
 	DBG("%p", mux);
 
-	if (!mux) {
+	if (mux == NULL) {
 		ofono_modem_set_powered(modem, FALSE);
 		return;
 	}
@@ -311,7 +460,7 @@ static int phonesim_enable(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	address = ofono_modem_get_string(modem, "Address");
-	if (!address)
+	if (address == NULL)
 		return -EINVAL;
 
 	port = ofono_modem_get_integer(modem, "Port");
@@ -342,7 +491,7 @@ static int phonesim_enable(struct ofono_modem *modem)
 	}
 
 	io = g_io_channel_unix_new(sk);
-	if (!io) {
+	if (io == NULL) {
 		close(sk);
 		return -ENOMEM;
 	}
@@ -357,7 +506,7 @@ static int phonesim_enable(struct ofono_modem *modem)
 	g_at_syntax_unref(syntax);
 	g_io_channel_unref(io);
 
-	if (!data->chat)
+	if (data->chat == NULL)
 		return -ENOMEM;
 
 	if (getenv("OFONO_AT_DEBUG"))
@@ -389,6 +538,9 @@ static int phonesim_enable(struct ofono_modem *modem)
 	g_at_chat_send(data->chat, "AT+CSCS=\"GSM\"", none_prefix,
 			NULL, NULL, NULL);
 
+	g_at_chat_register(data->chat, "+CRST:",
+				crst_notify, FALSE, modem, NULL);
+
 	return 0;
 }
 
@@ -465,11 +617,17 @@ static void phonesim_post_sim(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
+	ofono_ctm_create(modem, 0, "phonesim", data->chat);
 	ofono_phonebook_create(modem, 0, "atmodem", data->chat);
 
 	if (!data->calypso)
 		ofono_stk_create(modem, OFONO_VENDOR_PHONESIM,
 					"atmodem", data->chat);
+
+	ofono_call_forwarding_create(modem, 0, "atmodem", data->chat);
+
+	if (!data->calypso)
+		ofono_sms_create(modem, 0, "atmodem", data->chat);
 }
 
 static void phonesim_post_online(struct ofono_modem *modem)
@@ -482,7 +640,6 @@ static void phonesim_post_online(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	ofono_ussd_create(modem, 0, "atmodem", data->chat);
-	ofono_call_forwarding_create(modem, 0, "atmodem", data->chat);
 	ofono_call_settings_create(modem, 0, "atmodem", data->chat);
 
 	if (data->calypso)
@@ -497,10 +654,8 @@ static void phonesim_post_online(struct ofono_modem *modem)
 	ofono_ssn_create(modem, 0, "atmodem", data->chat);
 	ofono_call_volume_create(modem, 0, "atmodem", data->chat);
 
-	if (!data->calypso) {
-		ofono_sms_create(modem, 0, "atmodem", data->chat);
+	if (!data->calypso)
 		ofono_cbs_create(modem, 0, "atmodem", data->chat);
-	}
 
 	gprs = ofono_gprs_create(modem, 0, "atmodem", data->chat);
 
@@ -538,18 +693,18 @@ static struct ofono_modem *create_modem(GKeyFile *keyfile, const char *group)
 	DBG("group %s", group);
 
 	modem = ofono_modem_create(group, "phonesim");
-	if (!modem)
+	if (modem == NULL)
 		return NULL;
 
 	value = g_key_file_get_string(keyfile, group, "Address", NULL);
-	if (!value)
+	if (value == NULL)
 		goto error;
 
 	ofono_modem_set_string(modem, "Address", value);
 	g_free(value);
 
 	value = g_key_file_get_string(keyfile, group, "Port", NULL);
-	if (!value)
+	if (value == NULL)
 		goto error;
 
 	ofono_modem_set_integer(modem, "Port", atoi(value));
@@ -606,7 +761,7 @@ static void parse_config(const char *filename)
 		struct ofono_modem *modem;
 
 		modem = create_modem(keyfile, modems[i]);
-		if (!modem)
+		if (modem == NULL)
 			continue;
 
 		modem_list = g_slist_prepend(modem_list, modem);
@@ -630,6 +785,8 @@ static int phonesim_init(void)
 
 	ofono_gprs_context_driver_register(&context_driver);
 
+	ofono_ctm_driver_register(&ctm_driver);
+
 	parse_config(CONFIGDIR "/phonesim.conf");
 
 	return 0;
@@ -648,6 +805,8 @@ static void phonesim_exit(void)
 	g_slist_free(modem_list);
 	modem_list = NULL;
 
+	ofono_ctm_driver_unregister(&ctm_driver);
+
 	ofono_gprs_context_driver_unregister(&context_driver);
 
 	ofono_modem_driver_unregister(&phonesim_driver);
