@@ -25,12 +25,12 @@
 
 #include <ofono/types.h>
 
-void __ofono_exit();
+void __ofono_exit(void);
 
-int __ofono_manager_init();
-void __ofono_manager_cleanup();
+int __ofono_manager_init(void);
+void __ofono_manager_cleanup(void);
 
-void __ofono_modem_shutdown();
+void __ofono_modem_shutdown(void);
 
 #include <ofono/log.h>
 
@@ -103,30 +103,34 @@ void __ofono_modem_append_properties(struct ofono_modem *modem,
 struct ofono_atom;
 
 enum ofono_atom_type {
-	OFONO_ATOM_TYPE_DEVINFO = 0,
-	OFONO_ATOM_TYPE_CALL_BARRING = 1,
-	OFONO_ATOM_TYPE_CALL_FORWARDING = 2,
-	OFONO_ATOM_TYPE_CALL_METER = 3,
-	OFONO_ATOM_TYPE_CALL_SETTINGS = 4,
-	OFONO_ATOM_TYPE_NETREG = 5,
-	OFONO_ATOM_TYPE_PHONEBOOK = 6,
-	OFONO_ATOM_TYPE_SMS = 7,
-	OFONO_ATOM_TYPE_SIM = 8,
-	OFONO_ATOM_TYPE_USSD = 9,
-	OFONO_ATOM_TYPE_VOICECALL = 10,
-	OFONO_ATOM_TYPE_HISTORY = 11,
-	OFONO_ATOM_TYPE_SSN = 12,
-	OFONO_ATOM_TYPE_MESSAGE_WAITING = 13,
-	OFONO_ATOM_TYPE_CBS = 14,
-	OFONO_ATOM_TYPES_CALL_VOLUME = 15,
-	OFONO_ATOM_TYPE_GPRS = 16,
-	OFONO_ATOM_TYPE_GPRS_CONTEXT = 17,
-	OFONO_ATOM_TYPE_RADIO_SETTINGS = 18,
-	OFONO_ATOM_TYPE_AUDIO_SETTINGS = 19,
-	OFONO_ATOM_TYPE_STK = 20,
-	OFONO_ATOM_TYPE_NETTIME = 21,
-	OFONO_ATOM_TYPE_CTM = 22,
-	OFONO_ATOM_TYPE_CDMA_VOICECALL_MANAGER = 23,
+	OFONO_ATOM_TYPE_DEVINFO,
+	OFONO_ATOM_TYPE_CALL_BARRING,
+	OFONO_ATOM_TYPE_CALL_FORWARDING,
+	OFONO_ATOM_TYPE_CALL_METER,
+	OFONO_ATOM_TYPE_CALL_SETTINGS,
+	OFONO_ATOM_TYPE_NETREG,
+	OFONO_ATOM_TYPE_PHONEBOOK,
+	OFONO_ATOM_TYPE_SMS,
+	OFONO_ATOM_TYPE_SIM,
+	OFONO_ATOM_TYPE_USSD,
+	OFONO_ATOM_TYPE_VOICECALL,
+	OFONO_ATOM_TYPE_HISTORY,
+	OFONO_ATOM_TYPE_SSN,
+	OFONO_ATOM_TYPE_MESSAGE_WAITING,
+	OFONO_ATOM_TYPE_CBS,
+	OFONO_ATOM_TYPES_CALL_VOLUME,
+	OFONO_ATOM_TYPE_GPRS,
+	OFONO_ATOM_TYPE_GPRS_CONTEXT,
+	OFONO_ATOM_TYPE_RADIO_SETTINGS,
+	OFONO_ATOM_TYPE_AUDIO_SETTINGS,
+	OFONO_ATOM_TYPE_STK,
+	OFONO_ATOM_TYPE_NETTIME,
+	OFONO_ATOM_TYPE_CTM,
+	OFONO_ATOM_TYPE_CDMA_VOICECALL_MANAGER,
+	OFONO_ATOM_TYPE_SIM_AUTH,
+	OFONO_ATOM_TYPE_EMULATOR_DUN,
+	OFONO_ATOM_TYPE_EMULATOR_HFP,
+	OFONO_ATOM_TYPE_LOCATION_REPORTING,
 };
 
 enum ofono_atom_watch_condition {
@@ -145,6 +149,11 @@ struct ofono_atom *__ofono_modem_add_atom(struct ofono_modem *modem,
 					void (*destruct)(struct ofono_atom *),
 					void *data);
 
+struct ofono_atom *__ofono_modem_add_atom_offline(struct ofono_modem *modem,
+					enum ofono_atom_type type,
+					void (*destruct)(struct ofono_atom *),
+					void *data);
+
 struct ofono_atom *__ofono_modem_find_atom(struct ofono_modem *modem,
 						enum ofono_atom_type type);
 
@@ -174,19 +183,33 @@ void __ofono_atom_free(struct ofono_atom *atom);
 
 typedef void (*ofono_modemwatch_cb_t)(struct ofono_modem *modem,
 					gboolean added, void *data);
-void __ofono_modemwatch_init();
-void __ofono_modemwatch_cleanup();
+void __ofono_modemwatch_init(void);
+void __ofono_modemwatch_cleanup(void);
 unsigned int __ofono_modemwatch_add(ofono_modemwatch_cb_t cb, void *user,
 					ofono_destroy_func destroy);
 gboolean __ofono_modemwatch_remove(unsigned int id);
 
-typedef void (*ofono_modem_online_notify_func)(ofono_bool_t online, void *data);
+typedef void (*ofono_modem_online_notify_func)(struct ofono_modem *modem,
+						ofono_bool_t online,
+						void *data);
 unsigned int __ofono_modem_add_online_watch(struct ofono_modem *modem,
 					ofono_modem_online_notify_func notify,
 					void *data, ofono_destroy_func destroy);
 void __ofono_modem_remove_online_watch(struct ofono_modem *modem,
 					unsigned int id);
 
+typedef void (*ofono_modem_powered_notify_func)(struct ofono_modem *modem,
+						ofono_bool_t powered,
+						void *data);
+
+unsigned int __ofono_modem_add_powered_watch(struct ofono_modem *modem,
+					ofono_modem_online_notify_func notify,
+					void *data, ofono_destroy_func destroy);
+void __ofono_modem_remove_powered_watch(struct ofono_modem *modem,
+					unsigned int id);
+
+void __ofono_modem_sim_reset(struct ofono_modem *modem);
+
 #include <ofono/call-barring.h>
 
 gboolean __ofono_call_barring_is_busy(struct ofono_call_barring *cb);
@@ -208,6 +231,7 @@ gboolean __ofono_call_settings_is_busy(struct ofono_call_settings *cs);
 #include <ofono/radio-settings.h>
 #include <ofono/audio-settings.h>
 #include <ofono/ctm.h>
+#include <ofono/location-reporting.h>
 
 #include <ofono/voicecall.h>
 
@@ -244,6 +268,7 @@ enum ofono_sms_submit_flag {
 	OFONO_SMS_SUBMIT_FLAG_RECORD_HISTORY =	0x2,
 	OFONO_SMS_SUBMIT_FLAG_RETRY =		0x4,
 	OFONO_SMS_SUBMIT_FLAG_EXPOSE_DBUS =	0x8,
+	OFONO_SMS_SUBMIT_FLAG_REUSE_UUID =	0x10,
 };
 
 typedef void (*ofono_sms_txq_submit_cb_t)(gboolean ok, void *data);
@@ -297,6 +322,17 @@ ofono_bool_t __ofono_sim_service_available(struct ofono_sim *sim,
 						int ust_service,
 						int sst_service);
 
+ofono_bool_t __ofono_is_valid_sim_pin(const char *pin,
+					enum ofono_sim_password_type type);
+
+ofono_bool_t __ofono_is_valid_net_pin(const char *pin);
+
+void __ofono_sim_refresh(struct ofono_sim *sim, GSList *file_list,
+				ofono_bool_t full_file_change,
+				ofono_bool_t naa_init);
+
+void __ofono_sim_recheck_pin(struct ofono_sim *sim);
+
 #include <ofono/stk.h>
 
 typedef void (*__ofono_sms_sim_download_cb_t)(ofono_bool_t ok,
@@ -418,3 +454,15 @@ void __ofono_nettime_info_received(struct ofono_modem *modem,
 					struct ofono_network_time *info);
 
 #include <ofono/cdma-voicecall.h>
+#include <ofono/sim-auth.h>
+
+#include <ofono/gprs-provision.h>
+ofono_bool_t __ofono_gprs_provision_get_settings(const char *mcc,
+				const char *mnc, const char *spn,
+				struct ofono_gprs_provision_data **settings,
+				int *count);
+void __ofono_gprs_provision_free_settings(
+				struct ofono_gprs_provision_data *settings,
+				int count);
+
+#include <ofono/emulator.h>
